/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "connection.h"
#include "caosdb/info/v1alpha1/main.grpc.pb.h"
#include <iostream>
#include <string>
#include <stdexcept>

namespace caosdb {
using caosdb::info::v1alpha1::GeneralInfoService;
using caosdb::info::v1alpha1::VersionInfo;
using caosdb::info::v1alpha1::GetVersionInfoRequest;
using caosdb::info::v1alpha1::GetVersionInfoResponse;

CaosDBConnectionConfig::CaosDBConnectionConfig(const std::string &host, int port) {
    this->host = host;
    this->port = port;
}

auto CaosDBConnectionConfig::getHost() -> std::string {
  return this->host;
}

auto CaosDBConnectionConfig::getPort() const -> int {
  return this->port;
}

auto operator << (std::ostream &out, const CaosDBConnectionConfig &config) -> std::ostream & {
  out << "CaosDBConnectionConfig(\"" << config.host << "\"," << config.port << ")";
  return out;
}

CaosDBConnection::CaosDBConnection(const CaosDBConnectionConfig &config) : config(config) {
  this->config = config;
  std::shared_ptr<grpc::Channel> channel = grpc::CreateChannel(this->config.getHost() + ":" + std::to_string(this->config.getPort()), grpc::InsecureChannelCredentials());
  this->stub_ = GeneralInfoService::NewStub(channel);
}

auto operator << (std::ostream &out, const CaosDBConnection &connection) -> std::ostream & {
  out << "CaosDBConnection(" << connection.config << ")";
  return out;
}

auto CaosDBConnection::getGeneralInfoService() -> GeneralInfoService::Stub & {
  return *stub_;
}

[[nodiscard]]
auto CaosDBConnection::getVersionInfo() const -> const VersionInfo & {
  const GetVersionInfoRequest request;
  GetVersionInfoResponse response;
  grpc::ClientContext context;
  const grpc::Status status = stub_->GetVersionInfo(&context, request, &response);

  if(!status.ok()) {
    throw std::runtime_error(status.error_message());
  }
  return response.version_info();
}

} // namespace caosdb
